package com.caimei.mapper;

import com.caimei.model.po.*;
import com.caimei.model.vo.AddressVo;
import com.caimei.model.vo.CartProductVo;
import com.caimei.model.vo.ShopVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/25
 */
@Mapper
public interface OrderSubmitMapper {
    /**
     * 根据商品集合查询供应商信息
     *
     * @param cartIds
     * @return
     */
    List<ShopVo> findShopByCartIds(@Param("cartIds") String[] cartIds);

    /**
     * 查询购物车商品信息
     *
     * @param shopId
     * @param cartIds
     * @return
     */
    List<CartProductVo> findByShopCartProduct(@Param("shopId") Integer shopId, @Param("cartIds") String[] cartIds);

    /**
     * 查询用户信息
     *
     * @param userId
     * @return
     */
    UserPo findUser(Integer userId);

    /**
     * 查询商品信息
     *
     * @param productId
     * @return
     */
    CmHeHeProductPo getProduct(Integer productId);

    /**
     * 删除购物车
     *
     * @param userId    用户id
     * @param productId 商品id
     * @param heUserId  分销者id
     */
    void deleteCartByProductId(@Param("userId") Integer userId, @Param("productId") Integer productId, @Param("heUserId") Integer heUserId);

    /**
     * 保存订单信息
     *
     * @param order
     */
    void insertOrder(CmOrderPo order);

    /**
     * 查询子订单编号
     *
     * @param orderId
     * @return
     */
    String findMaxShopOrderNo(Long orderId);

    /**
     * 保存子订单信息
     *
     * @param shopOrder
     */
    void insertShopOrder(CmShopOrderPo shopOrder);

    /**
     * 保存订单商品信息
     *
     * @param orderProduct
     */
    void insertOrderProduct(CmOrderProductPo orderProduct);

    /**
     * 保存订单阶梯信息
     *
     * @param ladderPrice
     */
    void insertOrderProductLadderPrice(OrderProductLadderPricePo ladderPrice);

    /**
     * 更新订单信息
     *
     * @param order
     */
    void updateOrder(CmOrderPo order);

    /**
     * 查询地址信息
     *
     * @param addressId
     * @return
     */
    AddressVo findByAddressId(Integer addressId);

    /**
     * 保存订单地址信息
     *
     * @param userInfo
     */
    void insertUserInfo(BpOrderUserInfoPo userInfo);

    /**
     * 根据商品id查询供应商信息
     *
     * @param productId
     * @return
     */
    ShopVo findShopByProductId(Integer productId);

    /**
     * 查询商品信息
     *
     * @param productId
     * @return
     */
    CartProductVo findProductById(Integer productId);
}
